﻿using Newtonsoft.Json.Linq;
using System;
using System.Data.SQLite;
using System.Diagnostics;


namespace kintone集計結果履歴登録
{
    internal class sqlite3_lib
    {

        readonly string db_name = "kintone_records.sqlite3";


        public void CreateTable(string table_name, JObject arrFields)
        {

            SQLiteConnection con = new SQLiteConnection($"Data Source={db_name};Version=3;");
            con.Open();

            SQLiteCommand com = new SQLiteCommand($"drop table if exists {table_name}", con);
            com.ExecuteNonQuery();

            try
            {
                string sql = $@"create table if not exists {table_name} (" + Environment.NewLine;

                int cnt = 0;
                foreach (var field in arrFields)
                {
                    string code = field.Value["code"].ToString();
                    string tp = field.Value["type"].ToString();

                    string field_type = "TEXT";
                    if (tp == "NUMBER" || tp == "__ID__" || tp == "RECORD_NUMBER")
                    {
                        field_type = "NUMBER";
                    }
                    else if (tp == "DATE" || tp == "DATETIME" || tp == "CREATED_TIME" || tp == "UPDATED_TIME")
                    {
                        field_type = "TIMESTAMP";
                    }

                    if (cnt == 0)
                    {
                        sql += $@"{code} {field_type}" + Environment.NewLine;
                    }
                    else
                    {
                        sql += $@", {code} {field_type}" + Environment.NewLine;
                    }

                    cnt++;
                }
                sql += ")";

                com = new SQLiteCommand(sql, con);
                var sdr = com.ExecuteNonQuery();

            }
            finally
            {
                con.Close();
            }

        }


        private JArray ConvertJArrayToSimplifiedJArray(JArray jsonArray)
        {
            JArray simplifiedJsonArray = new JArray();

            foreach (var item in jsonArray)
            {
                JObject simplifiedItem = new JObject();

                foreach (JProperty property in item.Children<JProperty>())
                {
                    simplifiedItem[property.Name] = property.Value["value"];
                }

                simplifiedJsonArray.Add(simplifiedItem);
            }

            return simplifiedJsonArray;
        }



        // 日本時間に文字列を変換して返す
        private string convDatatiemJP(string dt)
        {
            DateTime dt2 = DateTime.Parse(dt);

            return dt2.ToLocalTime().ToString("yyyy-MM-dd HH:mm:ss");
        }



        public void insertData(string table_name, JObject fields, JArray data)
        {

            SQLiteConnection con = new SQLiteConnection($"Data Source={db_name};Version=3;");
            con.Open();
            SQLiteCommand com = new SQLiteCommand($"delete from {table_name}", con);
            com.ExecuteNonQuery();

            var data2 = ConvertJArrayToSimplifiedJArray(data);

            foreach (JObject item in data2)
            {
                string fields_str = "";
                string values = "( ";
                foreach (var prop in fields)
                {
                    string code = prop.Value["code"].ToString();
                    string tp = prop.Value["type"].ToString();

                    string val = null;
                    try
                    {
                        if (tp == "DATETIME" || tp == "CREATED_TIME" || tp == "UPDATED_TIME")
                        {
                            val = convDatatiemJP((string)item[code]);
                        }
                        else if (tp == "MODIFIER" || tp == "CREATOR")
                        {
                            val = (string)item[code]["name"];
                        }
                        else if (tp == "FILE")
                        {
                            val = "";
                            foreach (var fname in item[code])
                            {
                                val += (string)fname["name"] + ", ";
                            }
                        }
                        else
                        {
                            val = (string)item[code];
                        }
                    }
                    catch (Exception e)
                    {
                        //Debug.WriteLine(e.Message);
                    }
                    fields_str += code + ",";
                    values += $"\'{val}\',";
                }
                fields_str = fields_str.Substring(0, fields_str.Length - 1);
                values = values.Substring(0, values.Length - 1) + " )";

                string sql = $@"insert into {table_name} ({fields_str}) values {values}";

                com = new SQLiteCommand(sql, con);
                var sdr = com.ExecuteNonQuery();

            }
            con.Close();

        }

        public int selectDataOne(string sql)
        {
            int ret = 0;
            SQLiteConnection con = new SQLiteConnection($"Data Source={db_name};Version=3;");
            con.Open();
            try
            {
                // SELECT文を実行してすべてのレコードを取得
                //string sql = "SELECT * FROM tbl480";
                SQLiteCommand com = new SQLiteCommand(sql, con);
                SQLiteDataReader sdr = com.ExecuteReader();

                while (sdr.Read())
                {
                    ret = sdr.GetInt32(0);
                    //Debug.WriteLine(sdr[0]);
                }

                sdr.Close();
            }
            finally
            {
                // 接続を閉じる
                con.Close();
            }

            return ret;
        }

    }
}
