﻿using Newtonsoft.Json.Linq;
using NLog;
using System.Diagnostics;
using System.Linq;
using System.Net.Http;
using System.Text;


namespace kintone_sql_editor
{
    internal class Kintone
    {

        private static Logger logger = LogManager.GetCurrentClassLogger();

        string domain;
        int space_id;
        int app_id;
        string api_token;
        string user_pw64;


        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="_domain"></param>
        /// <param name="_space_id"></param>
        /// <param name="_app_id"></param>
        /// <param name="_api_token"></param>
        /// <param name="_user_pw64"></param>
        public Kintone(string _domain, int _space_id, int _app_id, string _api_token, string _user_pw64)
        {
            domain = _domain;
            space_id = _space_id;
            app_id = _app_id;
            api_token = _api_token;
            user_pw64 = _user_pw64;
        }



        public JObject GetFieldsRaw()
        {
            var objResult = recOperation($"app/form/fields.json?app={app_id}", "get", null);

            //objResultにキー "message"が存在するかをチェック
            if (objResult.ContainsKey("message"))
            {
                //MessageBox.Show("kintoneがエラーを返しました");
                return null;
            }

            var objProperties = (JObject)objResult["properties"];
            System.IO.File.WriteAllText("fields.json", objProperties.ToString());


            return objProperties;
        }


        /// <summary>
        /// kintoneのレコードを再帰的に全件取得する
        /// </summary>
        public JArray GetAllRecords()
        {
            JArray arrRecords = new JArray();
            int pageSize = 500;
            int offset = 0;
            int totalCount = 0;

            do
            {
                string filterCond = $"limit {pageSize} offset {offset}";
                string sub_url = $"records.json?app={app_id}&totalCount=true&query={filterCond}";
                var objResult = recOperation(sub_url, "get", null);
                totalCount = (int)objResult["totalCount"];
                var records = (JArray)objResult["records"];
                arrRecords.Merge(records);
                offset += pageSize; // 次のページを取得するためにoffsetを更新
            } while (offset < totalCount);


            System.IO.File.WriteAllText("arrRecords.json", arrRecords.ToString());

            return arrRecords;
        }




        /// <summary>
        /// kintone API呼び出しの共通関数
        /// </summary>
        /// <param name="sub_url"></param>
        /// <param name="mode"></param>
        /// <param name="mainObj"></param>
        /// <returns></returns>
        public JObject recOperation(string sub_url, string mode, JObject mainObj)
        {
            logger.Debug(mainObj);
            var client = new HttpClient();
            client.DefaultRequestHeaders.Add("ContentType", "application/json");
            if (api_token == null)
            {
                client.DefaultRequestHeaders.Add("X-Cybozu-Authorization", user_pw64);
                logger.Debug("パスワード認証");
            }
            else
            {
                client.DefaultRequestHeaders.Add("X-Cybozu-API-Token", api_token);
                logger.Debug("APIトークン認証");
            }

            string guest_path = "";
            if (space_id != 0)
            {
                guest_path = $"guest/{space_id}/";
            }

            string url = $"https://{domain}.cybozu.com/k/{guest_path}v1/{sub_url}";
            StringContent content = null;
            if (mainObj != null)
            {
                content = new StringContent(mainObj.ToString(), Encoding.UTF8, @"application/json");
            }

            HttpResponseMessage response = null;
            if (mode == "post")
            {
                response = client.PostAsync(url, content).Result;
            }
            else if (mode == "put")
            {
                response = client.PutAsync(url, content).Result;
            }
            else if (mode == "get")
            {
                response = client.GetAsync(url).Result;
            }
            string result = response.Content.ReadAsStringAsync().Result;
            var objResult = JObject.Parse(result);

            logger.Debug(response.StatusCode);
            logger.Debug(objResult);

            return objResult;
        }


        public void upsertData(JObject obj, string keyField)
        {
            JObject ret = recOperation("record.json", "post", obj);
            Debug.WriteLine(ret);
            if (ret.ContainsKey("errors"))
            {
                var key_val = obj["record"][keyField]["value"];
                obj["record"] = new JObject(obj["record"].Children().Where(x => ((JProperty)x).Name != keyField));
                obj["updateKey"] = new JObject();
                obj["updateKey"]["field"] = keyField;
                obj["updateKey"]["value"] = key_val;

                var ret2 = recOperation("record.json", "put", obj);
                Debug.WriteLine("キーが重複しているのでupdateします");
                Debug.WriteLine(ret2);
            }

        }


        public void updateData(JObject obj)
        {
            var ret = recOperation("record.json", "put", obj);
            Debug.WriteLine(ret);
        }

    }
}
